@@ -4,10 +4,12 @@ Curve classes used in YARD.
 This package contains implementations for all the curves YARD can plot.
 At the time of writing, this includes:
 
-    - ROC curves
-    - CROC curves
-    - Precision-recall curves
-    - Accumulation curves
+    - ROC curves (`ROCCurve`)
+    - CROC curves (`CROCCurve`)
+    - Precision-recall curves (`PrecisionRecallCurve`)
+    - Sensitivity-specificity plots (`SensitivitySpecificityCurve`)
+    - Accumulation curves (`AccumulationCurve`)
+    - F-score curves (`FScoreCurve`)
 """
 
 __author__  = "Tamas Nepusz"
@@ -19,7 +21,7 @@ from bisect import bisect
 from itertools import izip
 from yard.data import BinaryConfusionMatrix, BinaryClassifierData
 from yard.transform import ExponentialTransformation
-from yard.utils import axis_label
+from yard.utils import axis_label, itersubclasses
 
 class Curve(object):
     """Class representing an arbitrary curve on a 2D space.
@@ -249,6 +251,50 @@ class Curve(object):
         self._points = [(x, transformation(y)) for x, y in self._points]
 
 
+class CurveFactory(object):
+    """Factory class to construct `Curve` instances from short identifiers.
+    
+    Short identifiers for curve types are typically used in the command-line
+    interface of `yard` to let the user specify which curve he/she wants to
+    plot. This factory class interprets the short identifiers and constructs
+    appropriate `Curve` instances.
+    """
+
+    @classmethod
+    def construct_from_name(cls, name, *args, **kwds):
+        """Constructs a curve from a short name used in command line arguments
+        across the whole ``yard`` package.
+        
+        `name` is matched against the ``identifier`` class-level properties of
+        all the subclasses of `Curve` to find the subclass to be constructed.
+        All the remaining arguments are passed on intact to the constructor of
+        the subclass. Returns a new instance of the found subclass, or raises
+        ``ValueError`` if an invalid name was given.
+        """
+        return cls.find_class_by_name(name)(*args, **kwds)
+
+    @staticmethod
+    def find_class_by_name(name):
+        """Finds the class corresponding to a given short name used in command
+        line arguments across the whole ``yard`` package.
+        
+        `name` is matched against the ``identifier`` class-level properties of
+        all the subclasses of `Curve` to find the subclass to be constructed.
+        Returns the found subclass (not an instance of it), or raises
+        ``ValueError`` if an invalid name was given.
+        """
+        name = name.lower()
+        for cls in itersubclasses(Curve):
+            if hasattr(cls, "identifier") and cls.identifier == name:
+                return cls
+        raise ValueError("no such curve type: %s" % name)
+
+    @staticmethod
+    def get_curve_names():
+        return sorted([cls.identifier for cls in itersubclasses(Curve)
+                if hasattr(cls, "identifier")])
+
+
 class BinaryClassifierPerformanceCurve(Curve):
     """Class representing a broad class of binary classifier performance
     curves.
@@ -257,7 +303,9 @@ class BinaryClassifierPerformanceCurve(Curve):
     and Y axes of the plot. If you are interested in ROC curves, see
     `ROCCurve`, which is a subclass of this class. If you are interested
     in precision-recall curves, see `PrecisionRecallCurve`, which is also
-    a subclass. Accumulation curves are implemented in `AccumulationCurve`.
+    a subclass. Accumulation curves are implemented in `AccumulationCurve`,
+    sensitivity-specificity plots are in `SensitivitySpecificityCurve`
+    etc.
     """
 
     def __init__(self, data, x_func, y_func):
@@ -358,6 +406,8 @@ class ROCCurve(BinaryClassifierPerformanceCurve):
     the false positive rate on the X axis.
     """
 
+    identifier = "roc"
+
     def __init__(self, data):
         """Constructs a ROC curve from the given dataset.
 
@@ -444,6 +494,8 @@ class PrecisionRecallCurve(BinaryClassifierPerformanceCurve):
     recall on the X axis.
     """
 
+    identifier = "pr"
+
     def __init__(self, data):
         """Constructs a precision-recall curve from the given dataset.
 
@@ -524,6 +576,35 @@ class PrecisionRecallCurve(BinaryClassifierPerformanceCurve):
 
 
 
+class SensitivitySpecificityCurve(BinaryClassifierPerformanceCurve):
+    """Class representing a sensitivity-specificity plot.
+    
+    A sensitivity-specificity curve plots the sensitivity on the Y axis
+    versus the specificity on the X axis.
+    """
+
+    identifier = "sespe"
+
+    def __init__(self, data):
+        """Constructs a sensitivity-specificity curve from the given dataset.
+
+        The dataset must contain ``(x, y)`` pairs where `x` is a predicted
+        value and `y` defines whether the example is positive or negative.
+        When `y` is less than or equal to zero, it is considered a negative
+        example, otherwise it is positive. ``False`` also means a negative
+        and ``True`` also means a positive example. The dataset can also
+        be an instance of `BinaryClassifierData`.
+        """
+        super(SensitivitySpecificityCurve, self).__init__(data,
+            BinaryConfusionMatrix.tnr, BinaryConfusionMatrix.recall)
+
+    @classmethod
+    def get_friendly_name(cls):
+        """Returns a human-readable name of the curve that can be
+        used in messages."""
+        return "sensitivity-specificity plot"
+
+
 class AccumulationCurve(BinaryClassifierPerformanceCurve):
     """Class representing an accumulation curve.
     
@@ -531,6 +612,8 @@ class AccumulationCurve(BinaryClassifierPerformanceCurve):
     versus the fraction of data classified as positive on the X axis.
     """
 
+    identifier = "ac"
+
     def __init__(self, data):
         """Constructs an accumulation curve from the given dataset.
 
@@ -560,6 +643,8 @@ class CROCCurve(BinaryClassifierPerformanceCurve):
     (close to zero).
     """
 
+    identifier = "croc"
+
     def __init__(self, data, alpha = 7):
         """Constructs a CROC curve from the given dataset.
 
@@ -704,3 +789,43 @@ class CROCCurve(BinaryClassifierPerformanceCurve):
         return "concentrated ROC curve"
 
 
+class FScoreCurve(BinaryClassifierPerformanceCurve):
+    """Class representing an F-score curve.
+    
+    An F-score curve plots the F-score on the Y axis versus the fraction
+    of data classified as positive on the X axis.
+    """
+
+    identifier = "fscore"
+
+    def __init__(self, data, f=1.0):
+        """Constructs an F-score curve from the given dataset.
+
+        The dataset must contain ``(x, y)`` pairs where `x` is a predicted
+        value and `y` defines whether the example is positive or negative.
+        When `y` is less than or equal to zero, it is considered a negative
+        example, otherwise it is positive. ``False`` also means a negative
+        and ``True`` also means a positive example. The dataset can also
+        be an instance of `BinaryClassifierData`.
+
+        The value of `f` controls the weighting between precision and recall
+        in the F-score formula. `f` = 1 means that equal importance is attached
+        to precision and recall. In general, recall is considered `f` times more
+        important than precision.
+        """
+        @axis_label("F-score")
+        def f_score(matrix):
+            """Internal function that binds the `f` parameter of
+            `BinaryConfusionMatrix.f_score` to the value specified in the constructor.
+            """
+            return BinaryConfusionMatrix.f_score(matrix, f)
+        super(FScoreCurve, self).__init__(data, BinaryConfusionMatrix.fdp, f_score)
+
+    @classmethod
+    def get_friendly_name(cls):
+        """Returns a human-readable name of the curve that can be
+        used in messages."""
+        return "F-score curve"
+
+
+
